package cn.com.qmth.examcloud.task.service.job;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordForMarkingBean;
import cn.com.qmth.examcloud.marking.api.MarkWorkCloudService;
import cn.com.qmth.examcloud.marking.api.bean.MarkRangeBean;
import cn.com.qmth.examcloud.marking.api.bean.MarkWorkBean;
import cn.com.qmth.examcloud.marking.api.bean.MarkWorkExamBean;
import cn.com.qmth.examcloud.marking.api.bean.MarkWorkMainBean;
import cn.com.qmth.examcloud.marking.api.request.GetMarkWorkExamReq;
import cn.com.qmth.examcloud.marking.api.request.UpdateMarkWorkExamDataEndReq;
import cn.com.qmth.examcloud.marking.api.request.UpdateMarkWorkReq;
import cn.com.qmth.examcloud.marking.api.response.GetMarkWorkExamResp;
import cn.com.qmth.examcloud.marking.api.response.GetTodoMarkWorkMainResp;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.helper.ExamCacheTransferHelper;
import cn.com.qmth.examcloud.task.service.consumer.MarkWorkCreateConsumer;
import cn.com.qmth.examcloud.task.service.consumer.MarkWorkCreateItemsConsumer;
import cn.com.qmth.examcloud.task.service.consumer.MarkWorkCreateRangeConsumer;
import cn.com.qmth.examcloud.task.service.producer.MarkWorkCreateItemsProducer;
import cn.com.qmth.examcloud.task.service.producer.MarkWorkCreateProducer;
import cn.com.qmth.examcloud.task.service.producer.MarkWorkCreateRangeProducer;
import cn.com.qmth.examcloud.web.task.AbstractTask;
import cn.com.qmth.examcloud.web.task.ScheduleJob;
import cn.com.qmth.examcloud.web.task.TaskTracker;

@Component("markWorkPaperCreateTask")
public class MarkWorkPaperCreateTask extends AbstractTask {

    @Autowired
    private MarkWorkCloudService markWorkCloudService;

    @Autowired
    TaskTracker TaskTracker;

    @Override
    public void run(ScheduleJob scheduleJob) throws Exception {
        // 该任务只获取非千卷的basepaperid来创建item，千卷的不用创建
        // 运行一次处理完所有待处理的评卷任务
        for (;;) {
            GetTodoMarkWorkMainResp res = markWorkCloudService.getTodoMarkWorkMain();
            MarkWorkMainBean bean = res.getBean();
            if (res.getBean() == null) {
                break;
            }

            UpdateMarkWorkReq req = new UpdateMarkWorkReq();
            MarkWorkBean wbean = new MarkWorkBean();
            wbean.setId(bean.getId());
            wbean.setName(bean.getName());
            wbean.setRemark(bean.getRemark());
            wbean.setAppendTime(bean.getAppendTime());
            req.setMarkWorkBean(wbean);

            GetMarkWorkExamReq we = new GetMarkWorkExamReq();
            we.setWorkId(bean.getId());
            GetMarkWorkExamResp weres = markWorkCloudService.getMarkWorkExam(we);
            Map<Long, Boolean> map = new HashMap<>();
            for (MarkWorkExamBean web : weres.getData()) {
                map.put(web.getExamId(), web.getDataEnd());
            }

            try {
                for (Long examId : bean.getExamIds()) {
                    boolean dataEnd = false;
                    if (!map.get(examId)) {
                        ExamSettingsCacheBean eb = ExamCacheTransferHelper.getDefaultCachedExam(examId);
                        if (eb.getEndTime().getTime() <= System.currentTimeMillis()) {
                            dataEnd = true;
                        }
                        List<ExamRecordForMarkingBean> markRangeData = createMarkItems(examId, bean.getId());
                        createStudentPapers(examId, bean.getId());
                        createMarkRange(markRangeData);
                        if (dataEnd) {
                            markWorkExamDataEnd(bean.getId(), examId);
                        }
                    }
                }
                wbean.setStatus(1);
            } catch (StatusException e) {
                wbean.setStatus(2);
                throw e;
            } catch (Exception e) {
                wbean.setStatus(2);
                throw new StatusException("100001", "评卷工作试卷生成出错 ", e);
            } finally {
                markWorkCloudService.updateMarkWork(req);
            }
        }
    }

    private void markWorkExamDataEnd(Long workId, Long examId) {
        UpdateMarkWorkExamDataEndReq req = new UpdateMarkWorkExamDataEndReq(workId, examId);
        markWorkCloudService.updateMarkWorkExamDataEnd(req);
    }

    @SuppressWarnings("unchecked")
    private List<ExamRecordForMarkingBean> createMarkItems(Long examId, Long workId)
            throws InstantiationException, IllegalAccessException {
        MarkWorkCreateItemsProducer producer = new MarkWorkCreateItemsProducer();
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("examId", examId);
        param.put("workId", workId);
        return (List<ExamRecordForMarkingBean>) producer.startDispose(MarkWorkCreateItemsConsumer.class, 10, param);
    }

    private void createStudentPapers(Long examId, Long workId) throws InstantiationException, IllegalAccessException {
        MarkWorkCreateProducer producer = new MarkWorkCreateProducer();
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("examId", examId);
        param.put("workId", workId);
        producer.startDispose(MarkWorkCreateConsumer.class, 10, param);
    }

    private void createMarkRange(List<ExamRecordForMarkingBean> beans)
            throws InstantiationException, IllegalAccessException {
        List<MarkRangeBean> data = new ArrayList<>();
        Set<String> set = new HashSet<>();
        for (ExamRecordForMarkingBean bean : beans) {
            if (!set.contains(bean.getBasePaperId())) {
                set.add(bean.getBasePaperId());
                MarkRangeBean mb = new MarkRangeBean();
                mb.setBasePaperId(bean.getBasePaperId());
                mb.setCourseId(bean.getCourseId());
                mb.setRandomPaper(bean.getRandomPaper());
                data.add(mb);
            }
        }

        MarkWorkCreateRangeProducer producer = new MarkWorkCreateRangeProducer();
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("data", data);
        producer.startDispose(MarkWorkCreateRangeConsumer.class, 10, param);
    }

    @Override
    public TaskTracker getTaskTracker() {
        return TaskTracker;
    }
}
